!****************************************************************************
!
!  PROGRAM: Estimates the EK model parameters with OECD data
!
!****************************************************************************

program main
	use MSIMSL

	implicit none
	
	! data variables
	! ==========================================================	
	integer(4), parameter :: NN = 19				! number of countries
	real(8), dimension(NN) :: H						! population in millions
	real(8), dimension(NN) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN) :: wpop					! working population, as a faction of total (inactive in main specificaiton)
	real(8), dimension(NN, NN) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN) :: distance			! distance in km
	integer(4), dimension(NN, NN) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN) :: agree			! 1 for both countries in the same trade agreement
	real(8) :: uswage
	
	! polytope variables

	integer(4), parameter :: n = 6					! number of parameters to estimate
	real(8), dimension(n), parameter :: xup = (/ 2.3, .4, -0.001, 1.2, 1.2, 1.2 /), &	! upper and lower bounds for initial parameter guesses
										xlow = (/ 1.5, .1, -0.05, 0.8, 0.8, 0.8 /)
	integer(4), parameter :: np1 = n + 1	
	integer(4), parameter :: two = 2
	real(8), parameter :: usual_delta = 0.1, zero_term_delta = 0.00025, &
							prho = 1., pchi = 2., ppsi = 0.5, psigma = 0.5, &
							tol_fun = 5e-3, tol_par = 1e-2, &		!tolerance within polytope
							ptol_fun = 5e-3, ptol_par = 1e-2		!tolerance comparing polytopes
	integer(4), parameter ::  maxfun = 200*n, maxiter = 200*n 
	integer(4), parameter:: maxj = 50, max_xiguess = 10					!runs at most 50 polytopes per initial guess
	real(8), dimension(n, np1) :: v
	real(8), dimension(np1) :: fv
	real(8), dimension(n) :: xinitial
	integer :: startime, endtime, hz
	integer(4) :: j, xiguess, imp, sizeout

	! model variables/parameters
	! ================================================================
	real(8), dimension(NN) :: T
	logical :: succt
	real(8) :: obj, weight
	real(8), dimension(:), allocatable :: output

	! ================================================================
	CALL SYSTEM_CLOCK(COUNT_RATE=hz)
	CALL SYSTEM_CLOCK(COUNT=startime)

	! ======================================================================
	! read data

	call readata(NN, H, w, wpop, XT99, distance, contig, lang, agree, uswage)

	! =======================================================================
	! read initial parameters

	open(2, file = 'init_paramekoecd.txt', STATUS ='OLD')
		do j = 1, n
			read(2,*), xinitial(j)
		end do
	close(2)

	! read initial guess of T
	open(2, file = 'init_techekoecd.txt', STATUS ='OLD')
		do j = 1, NN
			read(2,*), T(j)
		end do
	close(2)
	T = T/T(1)

! ===================================================================================
! calculates objective function with initial parameter guess

	call calc_obj(NN, n, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, succt, xinitial, obj)
	print *, " ========================================================================= "
	print *, xinitial
	print *, succt
	print *, obj

	CALL SYSTEM_CLOCK(COUNT=endtime)

	print*, " "
	print*, "=========================================================================="
	print*, "Time: ", DBLE(endtime - startime)/(60*DBLE(hz)), " minutes"
	print*, " "
	print*, " "

! ======================================================================================
! ONE POLYTOPE CYCLE IS RUN
! ======================================================================================
call polytope(n, np1, two, usual_delta, zero_term_delta, prho, pchi, ppsi, psigma, tol_fun, &
				tol_par, ptol_fun, ptol_par, maxfun, maxiter, maxj, xinitial, v, fv, &
				NN, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T)

	! ==========================================================================
	! save results

	1 Format(f13.9) 
	open(1, FILE='result_ek.txt', STATUS='REPLACE')
		write(1,*), fv(1)
		write(1,*), ' '
	do j = 1, n
			write(1,*), v(j,1)
		end do

		write(1,*), ' '
		do j = 1, NN
			write(1,*), T(j)
		end do
	close(1)

	CALL SYSTEM_CLOCK(COUNT=endtime)
	print*, " "
	print*, "=========================================================================="
	print*, "Time: ", DBLE(endtime - startime)/(60*DBLE(hz)), " minutes"
	print*, " "
	print*, " " 

! ======================================================================================
! SEVERAL (MAXIGUESS) POLYTOPE CYCLES ARE RUN
! ======================================================================================

xiguess = 1

do
	if (xiguess > max_xiguess) exit

	! construct new guess
	! =========================================================

!	startx : do
		do j = 1, n
			call random_number(xinitial(j))
		end do
		xinitial = (xinitial * (xup - xlow)) + xlow

		open(2, file = 'init_techun.txt', STATUS ='OLD')
			do j = 1, NN
				read(2,*), T(j)
			end do
		close(2)
		call calc_obj(NN, n, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, succt, xinitial, obj)
			print *, xinitial
			print *, succt
			print *, obj

	call polytope(n, np1, two, usual_delta, zero_term_delta, prho, pchi, ppsi, psigma, tol_fun, &
					tol_par, ptol_fun, ptol_par, maxfun, maxiter, maxj, xinitial, v, fv, &
					NN, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T)

	! save results
	! =========================================================
	
	select case(xiguess)

		case(1)
			open(1, FILE='result_ek1.txt', STATUS='REPLACE')
		case(2)
			open(1, FILE='result_ek2.txt', STATUS='REPLACE')
		case(3)
			open(1, FILE='result_ek3.txt', STATUS='REPLACE')
		case(4)
			open(1, FILE='result_ek4.txt', STATUS='REPLACE')
		case(5)
			open(1, FILE='result_ek5.txt', STATUS='REPLACE')
		case(6)
			open(1, FILE='result_ek6.txt', STATUS='REPLACE')
		case(7)
			open(1, FILE='result_ek7.txt', STATUS='REPLACE')
		case(8)
			open(1, FILE='result_ek8.txt', STATUS='REPLACE')
		case(9)
			open(1, FILE='result_ek9.txt', STATUS='REPLACE')
		case(10)
			open(1, FILE='result_ek10.txt', STATUS='REPLACE')

	end select
	
	write(1,*), fv(1)
	write(1,*), ' '
	
	do j = 1, n
		write(1,*), v(j,1)
	end do
	write(1,*), ' '

	do j = 1, NN
		write(1,*), T(j)
	end do

	close(1)
	! ================================================================
	
	xiguess = xiguess + 1

	CALL SYSTEM_CLOCK(COUNT=endtime)

	print*, " "
	print*, "=========================================================================="
	print*, "Time: ", DBLE(endtime - startime)/(60*DBLE(hz)), " minutes"
	print*, " "
	print*, " "	

end do	


end program main

