!****************************************************************************
!
!  PROGRAM: newmodel_ineqno
!
!  PURPOSE:  Entry point for the console application.
!			 estimates the new model with only with the set of countries that have income distribution data
!			 income inequality within countries is considered when calculating demand only
!
!****************************************************************************

program main
	use MSIMSL

	implicit none
	
	! data variables
	! ==========================================================	
	integer(4), parameter :: NN = 121				! number of countries
	integer(4), parameter :: fiveNN = NN*5			! number of countries
	real(8), dimension(NN) :: H						! population in millions
	real(8), dimension(NN) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN) :: wpop					! working population (as a faction of total)
	real(8), dimension(NN, NN) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN) :: distance			! distance in km
	integer(4), dimension(NN, NN) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN) :: agree			! 1 for both countries in the same trade agreement--ASEAN, COMESA, EFTA, MERCOSUR, NAFTA, EU
	real(8) :: uswage
	real(8), dimension(NN) :: w_highest, w_fourth, w_third, w_second, w_lowest

	! polytope variables
	
	integer(4), parameter :: n = 9			!number of variables to choose
	
	real(8), dimension(n), parameter :: xup = (/2., .3, -0.001, 0.9, 0.9, 0.9, 1.1, 17., .15/), &
										xlow = (/ 1.3, .1, -0.03, 0.7, 0.7, 0.7, .5, 10., .01 /)

	integer(4), parameter :: np1 = n + 1	
	integer(4), parameter :: two = 2
	real(8), parameter :: usual_delta = 0.07, zero_term_delta = 0.00025, &
							prho = 1., pchi = 2., ppsi = 0.5, psigma = 0.5, &
							tol_fun = 5e-3, tol_par = 1e-2, &		!tolerance within polytope
							ptol_fun = 5e-3, ptol_par = 1e-2		!tolerance comparing polytopes
	integer(4), parameter ::  maxfun = 200*n, maxiter = 200*n ! maxfun = 10, maxiter = 10 ! 
	integer(4), parameter:: maxj = 5, max_xiguess = 10					!runs at most 5 polytopes per initial guess
	real(8), dimension(n, np1) :: v
	real(8), dimension(np1) :: fv
	real(8), dimension(n) :: xinitial
	integer :: startime, endtime, hz
	integer(4) :: imp, expo, j, xiguess

	! model variables/parameters
	! ================================================================
	real(8), dimension(NN) :: T
	real(8), dimension(fiveNN) :: lambda
	logical :: succt
	real(8) :: obj
	
	! ================================================================
	CALL SYSTEM_CLOCK(COUNT_RATE=hz)
	CALL SYSTEM_CLOCK(COUNT=startime)

	! ======================================================================
	! read data

	call readata(NN, H, w, wpop, XT99, distance, contig, lang, agree, uswage, w_highest, w_fourth, w_third, w_second, w_lowest)

	! =======================================================================
	! read initial parameters

	open(2, file = 'init_paramun.txt', STATUS ='OLD')
		do j = 1, n
			read(2,*), xinitial(j)
		end do
	close(2)

	! read initial guess of T
	open(2, file = 'init_techineq.txt', STATUS ='OLD')
		do j = 1, NN
			read(2,*), T(j)
		end do
	close(2)

	! read initial guess of lambda
	open(2, file = 'init_lambdaineq.txt', STATUS ='OLD')
		do j = 1, fiveNN
			read(2,*), lambda(j)
		end do
	close(2)

! ===================================================================================
! calculate the objective function with initial parameter guess

	call calc_obj(NN, fiveNN, n, H, w, wpop, XT99, distance, contig, lang, agree, uswage, w_highest, w_fourth, w_third, w_second, w_lowest, T, lambda, succt, xinitial, obj)
	print *, xinitial
	print *, succt
	print *, obj

	open(1, FILE='init_techineq.txt', STATUS='REPLACE')
		do j = 1, NN
			write(1,*), T(j)
		end do
	close(1)

	open(1, FILE='init_lambda.txt', STATUS='REPLACE')
		do j = 1, fiveNN
			write(1,*), lambda(j)
		end do
	close(1)

	CALL SYSTEM_CLOCK(COUNT=endtime)

	print*, " "
	print*, "=========================================================================="
	print*, "Time: ", DBLE(endtime - startime)/(60*DBLE(hz)), " minutes"
	print*, " "
	print*, " " 

! ======================================================================================
! ONE POLYTOPE CYCLE IS RUN
! ======================================================================================
	call polytope(n, np1, two, usual_delta, zero_term_delta, prho, pchi, ppsi, psigma, tol_fun, &
					tol_par, ptol_fun, ptol_par, maxfun, maxiter, maxj, xinitial, v, fv, &
					NN, fiveNN, H, w, wpop, XT99, distance, contig, lang, agree, uswage, w_highest, w_fourth, w_third, w_second, w_lowest, T, lambda)

	! ==========================================================================
	! save results

	1 Format(f13.9) 

	open(1, FILE='result_ineq.txt', STATUS='REPLACE')
		write(1,*), fv(1)
		write(1,*), ' '

		do j = 1, n
			write(1,*), v(j,1)
		end do
		write(1,*), ' '
		write(1,*), "technologies"
		do j = 1, NN
			write(1,*), T(j)
		end do

		write(1,*), ' '
		write(1,*), "lambda"
		do j = 1, fiveNN
			write(1,*), lambda(j)
		end do
	close(1)

! ======================================================================================
! SEVERAL (MAXIGUESS) POLYTOPE CYCLES ARE RUN
! ======================================================================================

xiguess = 1

do j = 1, 3*n+1
	call random_number(xinitial(1))
end do

do
	if (xiguess >= max_xiguess) exit

	! construct new guess
	! =========================================================
	do j = 1, n
		call random_number(xinitial(j))
	end do
	xinitial = (xinitial * (xup - xlow)) + xlow

	! read initial guess of T
	! =========================================================
	open(2, file = 'init_techineqd.txt', STATUS ='OLD')
		do j = 1, NN
			read(2,*), T(j)
		end do
	close(2)

	call calc_obj(NN, fiveNN, n, H, w, wpop, XT99, distance, contig, lang, agree, uswage, w_highest, w_fourth, w_third, w_second, w_lowest, T, lambda, succt, xinitial, obj)
	print *, xinitial
	print *, succt
	print *, obj

	call polytope(n, np1, two, usual_delta, zero_term_delta, prho, pchi, ppsi, psigma, tol_fun, &
					tol_par, ptol_fun, ptol_par, maxfun, maxiter, maxj, xinitial, v, fv, &
					NN, fiveNN, H, w, wpop, XT99, distance, contig, lang, agree, uswage, w_highest, w_fourth, w_third, w_second, w_lowest, T, lambda)

	! save 
	! =========================================================
	
	select case(xiguess)

		case(1)
			open(1, FILE='result_ineqd1.txt', STATUS='REPLACE')
		case(2)
			open(1, FILE='result_ineqd2.txt', STATUS='REPLACE')
		case(3)
			open(1, FILE='result_ineqd3.txt', STATUS='REPLACE')
		case(4)
			open(1, FILE='result_ineqd4.txt', STATUS='REPLACE')
		case(5)
			open(1, FILE='result_ineqd5.txt', STATUS='REPLACE')
		case(6)
			open(1, FILE='result_ineqd6.txt', STATUS='REPLACE')
		case(7)
			open(1, FILE='result_ineqd7.txt', STATUS='REPLACE')
		case(8)
			open(1, FILE='result_ineqd8.txt', STATUS='REPLACE')
		case(9)
			open(1, FILE='result_ineqd9.txt', STATUS='REPLACE')
		case(10)
			open(1, FILE='result_ineqd10.txt', STATUS='REPLACE')

	end select
	
		write(1,*), fv(1)
		write(1,*), ' '
	
		do j = 1, n
			write(1,*), v(j,1)
		end do
		write(1,*), ' '
		write(1,*), "technologies"

		do j = 1, NN
			write(1,*), T(j)
		end do

		write(1,*), ' '
		write(1,*), "lambda"
		do j = 1, fiveNN
			write(1,*), lambda(j)
		end do

	close(1)
	! ================================================================
	
	xiguess = xiguess + 1

	CALL SYSTEM_CLOCK(COUNT=endtime)

	print*, " "
	print*, "=========================================================================="
	print*, "Time: ", DBLE(endtime - startime)/(60*DBLE(hz)), " minutes"
	print*, " "
	print*, " "	

end do	

end program main

