!****************************************************************************
!
!  PROGRAM: Monte Carlo
!
!****************************************************************************

program main
	use MSIMSL

	implicit none
	
	! data variables
	! ==========================================================	
	integer(4), parameter :: NN = 162				!number of countries
	real(8), dimension(NN) :: H						!population in millions
	real(8), dimension(NN) :: w						!wages 1999 (normalized to US = 1)
	real(8), dimension(NN) :: wpop					!working population (as a faction of total)
	real(8), dimension(NN, NN) :: XTdata			!trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN) :: distance			!distance in km
	integer(4), dimension(NN, NN) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN) :: agree			! 1 for both countries in the 
	integer(4), dimension(NN, NN) :: XTavail
	real(8) :: uswage

	! polytope variables
	
	integer(4), parameter :: n = 9					!number of variables to choose
	
	real(8), dimension(n), parameter :: xup = (/2., 0.5, -0.1, 1.2, 1.2, 1.2, 2., 19., .54/), &
										xlow = (/ 0.5, 0.05, 0., 0.5, 0.5, 0.5, .1, 4., .01 /)

	integer(4), parameter :: np1 = n + 1	
	integer(4), parameter :: two = 2
	real(8), parameter :: usual_delta = 0.07, zero_term_delta = 0.00025, &
							prho = 1., pchi = 2., ppsi = 0.5, psigma = 0.5, &
							tol_fun = 5e-3, tol_par = 1e-2, &		!tolerance within polytope
							ptol_fun = 5e-3, ptol_par = 1e-2		!tolerance comparing polytopes
	integer(4), parameter ::  maxsimul = 100, maxfun = 200*n, maxiter = 200*n ! maxfun = 10, maxiter = 10 ! 
	integer(4), parameter:: maxj = 5, maxguesses = 10						!runs at most 5 polytopes per initial guess
	real(8), dimension(n, np1) :: v
	real(8), dimension(np1) :: fv
	real(8), dimension(n) :: xdata, xinitial, randv, xmin
	integer :: startime, endtime, hz
	integer(4) :: j, nsimul, nfailures, quadrante, nguess

	! model variables/parameters
	! ================================================================
	real(8), dimension(NN) :: T, lambda
	logical :: succt, succdata
	real(8) :: obj, aux, objmin
	
	! ================================================================
	CALL SYSTEM_CLOCK(COUNT_RATE=hz)
	CALL SYSTEM_CLOCK(COUNT=startime)

	! ======================================================================
	! read data

	call readata(NN, H, w, wpop, XTdata, distance, contig, lang, agree, uswage)
	XTavail = 0
	where (XTdata < -.0000001) XTavail = -1
	XTdata = 0.

	! change the seed of the random draws (not necessary)
	do j = 1, 2*n+1
		call random_number(randv(1))
	end do

! ======================================================================================
! BEGIN MONTE CARLO CYCLE
! ======================================================================================

open(7, FILE='result_montecarloun_g10.txt', STATUS='REPLACE')

nsimul = 1
nfailures = 0
do j = 1, 31
	call random_number(randv(1))
end do
data_genloop: do
	if (nsimul > maxsimul) exit

	! generate data
	! =========================================================
	do j = 1, n
		call random_number(randv(j))
	end do
	xdata = (randv * (xup - xlow)) + xlow

	T = w**(2/(8.28 + xdata(9) + 1))

	call gen_data(NN, n, H, w, wpop, XTdata, distance, contig, lang, agree, uswage, T, lambda, succdata, xdata, quadrante)
	where (XTavail == -1) XTdata = -1.
	print *, " simulation number ", nsimul
	print *, " ============================================================ "
	print *, "data generation success", succdata
	print *, "data in quadrant", quadrante
	print "(f15.5, f15.5, f15.5, f15.5)", xdata
	print *, " " 

objmin = 10.
xmin = 10.
fv = 15.
nguess = 1

estimation_loop: do	
	
	! stop control due to data failure in data generating process
	if (.not. succdata) then 
		nfailures = 1 + nfailures
		exit estimation_loop
	else if (nguess == 1) then
		write(7,*), nsimul
			do j = 1, n
			write(7,*), xdata(j)
		end do
	end if
	
	! =========================================================
	print *, " ------------------------------------------------------------ "
	print *, "estimation #", nguess
	obj = 10.
	succt = .true.
	initialguess: do 
		if (obj < 2.) exit initialguess
		
		do j = 1, n
			call random_number(randv(j))
		end do
		xinitial = (randv * (xup - xlow)) + xlow
		if (.not. succt) T = w**(2/(8.28 + xinitial(9) + 1))

		call calc_obj(NN, n, H, w, wpop, XTdata, distance, contig, lang, agree, uswage, T, lambda, succt, xinitial, obj)
		print *, "obj, initial parameters", obj

	end do initialguess

	print "(f15.5, f15.5, f15.5, f15.5)", xinitial
	print *, " "
	if (succt) then
		call polytope(n, np1, two, usual_delta, zero_term_delta, prho, pchi, ppsi, psigma, tol_fun, &
					tol_par, ptol_fun, ptol_par, maxfun, maxiter, maxj, xinitial, v, fv, &
					NN, H, w, wpop, XTdata, distance, contig, lang, agree, uswage, T, lambda)
	else 
		print *, "estimation not successful"
		v = 0.
		fv = 15.
	end if
	
	!record results from all guesses
	j = 100*nsimul + nguess
	write(7,*), j	
	write(7,*), fv(1)
	do j = 1, n
		write(7,*), v(j,1)
	end do
	! keep smallest objective function
	if (fv(1) < objmin) then
		objmin = fv(1)
	end if
		
!	if ( (fv(1)<=0.00001) .or. (nguess >= maxguesses) ) exit estimation_loop
	if ( (nguess >= maxguesses) ) exit estimation_loop
	
nguess = nguess + 1
end do estimation_loop

nsimul = nsimul + 1

CALL SYSTEM_CLOCK(COUNT=endtime)

print*, " "
print*, "=========================================================================="
print*, "Time: ", DBLE(endtime - startime)/(60*DBLE(hz)), " minutes"
print*, " "

end do data_genloop

	write(7,*), "number of failed simulations"
	write(7,*), nfailures
close(7)

end program main

