!****************************************************************************
!
!  PROGRAM: Estimates the new model with specification 1, the full structural specification
!
!  PURPOSE:  Entry point for the console application.
!
!****************************************************************************

program main
	use MSIMSL

	implicit none
	
	! data variables
	! ==========================================================	
	integer(4), parameter :: NN = 162				! number of countries
	real(8), dimension(NN) :: H						! population in millions
	real(8), dimension(NN) :: w						! wages 2000 (normalized to US = 1, income per capita)
	real(8), dimension(NN) :: wpop					! working population as a faction of total (This variable is not used in the main specification)
	real(8), dimension(NN, NN) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN) :: distance			! distance in km
	integer(4), dimension(NN, NN) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN) :: agree			! 1 for both countries in the same trade agreement--ASEAN, COMESA, EFTA, MERCOSUR, NAFTA, EU
	real(8) :: uswage

	! polytope variables
	
	integer(4), parameter :: n = 9			! number of variables to be estimated
	
	real(8), dimension(n), parameter :: xup = (/1.6, .3, -0.001, 1.1, 1.1, 1.1, 1.1, 17., .15/), &	! upper and lower bounds for initial guesses of parameters
										xlow = (/ 1.2, .1, -0.03, 0.8, 0.8, 0.8, .5, 10., .01 /)

	integer(4), parameter :: np1 = n + 1	
	integer(4), parameter :: two = 2
	real(8), parameter :: usual_delta = 0.07, zero_term_delta = 0.00025, &
							prho = 1., pchi = 2., ppsi = 0.5, psigma = 0.5, &
							tol_fun = 5e-3, tol_par = 1e-2, &		! tolerance within polytope
							ptol_fun = 5e-3, ptol_par = 1e-2		! tolerance comparing polytopes
	integer(4), parameter ::  maxfun = 200*n, maxiter = 200*n		! maximum number of function evaluations
	integer(4), parameter:: maxj = 5, max_xiguess = 10				! runs at most 5 polytopes per initial guess
	real(8), dimension(n, np1) :: v
	real(8), dimension(np1) :: fv
	real(8), dimension(n) :: xinitial
	integer :: startime, endtime, hz
	integer(4) :: j, xiguess

	! model variables/parameters
	! ================================================================
	real(8), dimension(NN) :: T, lambda
	logical :: succt
	real(8) :: obj
	
	! ================================================================
	CALL SYSTEM_CLOCK(COUNT_RATE=hz)
	CALL SYSTEM_CLOCK(COUNT=startime)

	! ======================================================================
	! read data

	call readata(NN, H, w, wpop, XT99, distance, contig, lang, agree, uswage)

	! =======================================================================
	! read initial parameters

	open(2, file = 'init_paramun.txt', STATUS ='OLD')
		do j = 1, n
			read(2,*), xinitial(j)
		end do
	close(2)

	! read initial guess of T
	open(2, file = 'init_techun.txt', STATUS ='OLD')
		do j = 1, NN
			read(2,*), T(j)
		end do
	close(2)

	! read initial guess of lambda
	open(2, file = 'init_lambdaun.txt', STATUS ='OLD')
		do j = 1, NN
			read(2,*), lambda(j)
		end do
	close(2)

! ===================================================================================
! EVALUATION OF OBJECTIVE FUNCTION WITH INITIAL GUESS OF PARAMETERS

	call calc_obj(NN, n, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, lambda, succt, xinitial, obj)
	print *, xinitial
	print *, succt
	print *, obj

	open(1, FILE='init_techun.txt', STATUS='REPLACE')
		do j = 1, NN
			write(1,*), T(j)
		end do
	close(1)

	open(1, FILE='init_lambdaun.txt', STATUS='REPLACE')
		do j = 1, NN
			write(1,*), lambda(j)
		end do
	close(1)

	CALL SYSTEM_CLOCK(COUNT=endtime)

	print*, " "
	print*, "=========================================================================="
	print*, "Time: ", DBLE(endtime - startime)/(60*DBLE(hz)), " minutes"
	print*, " "
	print*, " " 

! ======================================================================================
! ONE POLYTOPE CYCLE IS RUN
! ======================================================================================
	call polytope(n, np1, two, usual_delta, zero_term_delta, prho, pchi, ppsi, psigma, tol_fun, &
					tol_par, ptol_fun, ptol_par, maxfun, maxiter, maxj, xinitial, v, fv, &
					NN, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, lambda)

	! ==========================================================================
	! save results

	1 Format(f13.9) 

	open(1, FILE='result_new1.txt', STATUS='REPLACE')
		write(1,*), fv(1)
		write(1,*), ' '

		do j = 1, n
			write(1,*), v(j,1)
		end do
		write(1,*), ' '
		write(1,*), "technologies"
		do j = 1, NN
			write(1,*), T(j)
		end do

		write(1,*), ' '
		write(1,*), "lambda"
		do j = 1, NN
			write(1,*), lambda(j)
		end do
	close(1)

! ======================================================================================
! SEVERAL (MAXIGUESS) POLYTOPE CYCLES ARE RUN
! ======================================================================================

xiguess = 1

do j = 1, 3*n+1
	call random_number(xinitial(1))
end do

do
	if (xiguess >= max_xiguess) exit

	! construct new guess
	! =========================================================
	do j = 1, n
		call random_number(xinitial(j))
	end do
	xinitial = (xinitial * (xup - xlow)) + xlow

	! read initial guess of T
	! =========================================================
	open(2, file = 'init_techun.txt', STATUS ='OLD')
		do j = 1, NN
			read(2,*), T(j)
		end do
	close(2)

	call calc_obj(NN, n, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, lambda, succt, xinitial, obj)
	print *, xinitial
	print *, succt
	print *, obj

	call polytope(n, np1, two, usual_delta, zero_term_delta, prho, pchi, ppsi, psigma, tol_fun, &
					tol_par, ptol_fun, ptol_par, maxfun, maxiter, maxj, xinitial, v, fv, &
					NN, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, lambda)

	! save 
	! =========================================================
	
	select case(xiguess)

		case(1)
			open(1, FILE='result_new11.txt', STATUS='REPLACE')
		case(2)
			open(1, FILE='result_new12.txt', STATUS='REPLACE')
		case(3)
			open(1, FILE='result_new13.txt', STATUS='REPLACE')
		case(4)
			open(1, FILE='result_new14.txt', STATUS='REPLACE')
		case(5)
			open(1, FILE='result_new15.txt', STATUS='REPLACE')
		case(6)
			open(1, FILE='result_new16.txt', STATUS='REPLACE')
		case(7)
			open(1, FILE='result_new17.txt', STATUS='REPLACE')
		case(8)
			open(1, FILE='result_new18.txt', STATUS='REPLACE')
		case(9)
			open(1, FILE='result_new19.txt', STATUS='REPLACE')
		case(10)
			open(1, FILE='result_new110.txt', STATUS='REPLACE')

	end select
	
		write(1,*), fv(1)
		write(1,*), ' '
	
		do j = 1, n
			write(1,*), v(j,1)
		end do
		write(1,*), ' '
		write(1,*), "technologies"

		do j = 1, NN
			write(1,*), T(j)
		end do

		write(1,*), ' '
		write(1,*), "lambda"
		do j = 1, NN
			write(1,*), lambda(j)
		end do

	close(1)
	! ================================================================
	
	xiguess = xiguess + 1

	CALL SYSTEM_CLOCK(COUNT=endtime)

	print*, " "
	print*, "=========================================================================="
	print*, "Time: ", DBLE(endtime - startime)/(60*DBLE(hz)), " minutes"
	print*, " "
	print*, " "	

end do	

end program main

