!****************************************************************************
!
!  PROGRAM: std_NLLS
!
!  PURPOSE:  Calculates standard errors given estimated parameters
!			 for the second empirical specification, with importer fixed effects
!****************************************************************************

program main
	use MSIMSL

	implicit none
	
	! data variables
	! ==========================================================	
	integer(4), parameter :: NN = 162				! number of countries
	real(8), dimension(NN) :: H						! population in millions
	real(8), dimension(NN) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN) :: wpop					! working population (as a faction of total)
	real(8), dimension(NN, NN) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN) :: distance			! distance in km
	integer(4), dimension(NN, NN) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN) :: agree			! 1 for both countries in the same trade agreement--ASEAN, COMESA, EFTA, MERCOSUR, NAFTA, EU
	real(8) :: uswage

	
	! model variables/parameters
	! ================================================================
	integer(4), parameter :: nshort = 7					! number of parameters
	integer(4), parameter :: n = nshort + NN					! number of parameters
	real(8), dimension(n) :: xoptimal
	integer(4) :: nobs, j, imp, expo				! nobs = number of observations
	real(8), dimension(n,n) :: varparam, clvarparam
	real(8), dimension(NN) :: T
	real(8), dimension(:), allocatable :: XTdata, XToptimal, residuals
	real(8), dimension(:,:), allocatable :: Xderivatives
	integer(4), dimension(:, :), allocatable :: obsimpexp
	logical :: succt
	real(8) :: obj
	
	real(8), parameter :: theta = 8.28
	
	! ======================================================================
	! read data

	call readata(NN, H, w, wpop, XT99, distance, contig, lang, agree, uswage)
	nobs = count(XT99>=-1e-7)
	nobs = nobs - NN
	print *, "number of observations = ", nobs
	print *, " "

	allocate (XTdata(nobs))
	allocate (XToptimal(nobs))
	allocate (residuals(nobs))
	allocate (Xderivatives(nobs, n))
	allocate (obsimpexp(nobs,2))
	
	! =======================================================================
	! read initial parameters

	open(2, file = 'optimal_paramFE.dat', STATUS ='OLD')
		do j = 1, n
			read(2,*), xoptimal(j)
		end do
	close(2)

	! read initial guess of T
	open(2, file = 'optimal_techFE.dat', STATUS ='OLD')
		do j = 1, NN
			read(2,*), T(j)
		end do
	close(2)

! ===================================================================================
! calculate objective function at optimal and store relevant data 

	call calc_optimal(NN, n, nobs, theta, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, succt, xoptimal, obj, XTdata, XToptimal, residuals, obsimpexp)

! ===================================================================================
! estimate matrix of derivatives with respect to parameters--d(X_ni/(X_nX_i))/d(xoptimal)
	
	call calc_derivatives(NN, n, nobs, theta, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, XToptimal, xoptimal, Xderivatives)

! ===================================================================================
! calculate variance of parameters 
	
!	case 1: no cluster
! ------------------------------------------------------
	call parameter_var(n, nobs, Xderivatives, residuals, varparam)

!   print parameters and corresponding variances:
	print *, " =================================== "
	print *, "parameter std. error without clustering (transformed parameters)"
	print *, "d1     ", xoptimal(1), varparam(1,1)**(.5)
	print *, "d2     ", xoptimal(2), varparam(2,2)**(.5)
	print *, "d3     ", xoptimal(3), varparam(3,3)**(.5)
	print *, "b      ", xoptimal(4), varparam(4,4)**(.5)
	print *, "l      ", xoptimal(5), varparam(5,5)**(.5)
	print *, "tagree ", xoptimal(6), varparam(6,6)**(.5)
	print *, "thetab ", xoptimal(7) + 1., varparam(7,7)**(.5)
	print *, " "

	!	case 2: cluster by exporter
	! ------------------------------------------------------
	call cluster_paramvar(NN, n, nobs, Xderivatives, residuals, obsimpexp, varparam)

	print *, " =================================== "
	print *, "parameter standard error  with clustering (transformed parameters)"
	print *, "d1     ", xoptimal(1), varparam(1,1)**.5
	print *, "d2     ", xoptimal(2), varparam(2,2)**.5
	print *, "d3     ", xoptimal(3), varparam(3,3)**.5
	print *, "b      ", xoptimal(4), varparam(4,4)**.5
	print *, "l      ", xoptimal(5), varparam(5,5)**.5
	print *, "tagree ", xoptimal(6), varparam(6,6)**.5
	print *, "thetab ", xoptimal(7) + 1., varparam(7,7)**(.5)
	print *, " "


	! ===================================================================================
	! get the variance of parameters using the delta method (instead of the variance-covariance matrix of their transforms)
	call deltavar(NN, n, xoptimal, varparam)

	print *, " =================================== "
	print *, "parameter standard error with clustering"
	print *, "d1     ", xoptimal(1), varparam(1,1)**.5
	print *, "d2     ", xoptimal(2), varparam(2,2)**.5
	print *, "d3     ", xoptimal(3), varparam(3,3)**.5
	print *, "b      ", xoptimal(4), varparam(4,4)**.5
	print *, "l      ", xoptimal(5), varparam(5,5)**.5
	print *, "tagree ", xoptimal(6), varparam(6,6)**.5
	print *, "thetab ", xoptimal(7) + 1., varparam(7,7)**(.5)
	print *, " "

	open(1, file = 'varparam_FE.csv', STATUS ='REPLACE')
		write(1,*), "parameter, value, standard error"
		write(1,*), "d1,     ", xoptimal(1), ", ", (varparam(1,1)**(.5))
		write(1,*), "d2,     ", xoptimal(2), ", ", (varparam(2,2)**(.5))
		write(1,*), "d3,     ", xoptimal(3), ", ", (varparam(3,3)**(.5))
		write(1,*), "b ,     ", xoptimal(4), ", ", (varparam(4,4)**(.5))
		write(1,*), "l ,     ", xoptimal(5), ", ", (varparam(5,5)**(.5))
		write(1,*), "tagree, ", xoptimal(6), ", ", (varparam(6,6)**(.5))
		write(1,*), "thetab, ", xoptimal(7) + 1., ", ", (varparam(7,7)**(.5))
		do j = n - NN +1, n
			write(1,*), "xsharea", ", ", xoptimal(j)/(1+xoptimal(j)), ", ", (varparam(j,j)**(.5))
		end do
	close(1)


end program main

