!****************************************************************************
!
!  PROGRAM: std_NLLS
!
!  PURPOSE:  Calculates standard errors, given estimated parameters, 
!			for the first empirical specification (the full structural model)
!****************************************************************************

program main
	use MSIMSL

	implicit none
	
	! data variables
	! ==========================================================	
	integer(4), parameter :: NN = 162				! number of countries
	real(8), dimension(NN) :: H						! population in millions
	real(8), dimension(NN) :: w						! wages 1999 (normalized to US = 1)
	real(8), dimension(NN) :: wpop					! working population (as a faction of total)
	real(8), dimension(NN, NN) :: XT99				! trade flows in millions (to be compatible with population), and divided by US wages 
	real(8), dimension(NN, NN) :: distance			! distance in km
	integer(4), dimension(NN, NN) :: contig			! 1 for contiguity
	integer(4), dimension(NN, NN) :: lang			! 1 for common official language
	integer(4), dimension(NN, NN) :: agree			! 1 for both countries in the same trade agreement--ASEAN, COMESA, EFTA, MERCOSUR, NAFTA, EU
	real(8) :: uswage

	
	! model variables/parameters
	! ================================================================
	integer(4), parameter :: n = 9					! number of parameters
	real(8), dimension(n) :: xoptimal
	integer(4) :: nobs, j, imp, expo				! nobs = number of observations
	real(8), dimension(n,n) :: varparam, clvarparam
	real(8), dimension(NN) :: T, lambda
	real(8), dimension(:), allocatable :: XTdata, XToptimal, residuals
	real(8), dimension(:,:), allocatable :: Xderivatives
	integer(4), dimension(:, :), allocatable :: obsimpexp
	logical :: succt
	real(8) :: obj
	integer :: startime, endtime, hz

	real(8), parameter :: theta = 8.28, &				!8.28, 3.60 and 12.86 are the estimates in EK
						  sigmaa = 5.0				! Broda and Weinstein

	! ================================================================
	CALL SYSTEM_CLOCK(COUNT_RATE=hz)
	CALL SYSTEM_CLOCK(COUNT=startime)

	! ======================================================================
	! read data

	call readata(NN, H, w, wpop, XT99, distance, contig, lang, agree, uswage)
	nobs = count(XT99>=-1e-7)
	nobs = nobs - NN
	print *, "number of observations = ", nobs
	print *, " "

	allocate (XTdata(nobs))
	allocate (XToptimal(nobs))
	allocate (residuals(nobs))
	allocate (Xderivatives(nobs, n))
	allocate (obsimpexp(nobs,2))
	
	! =======================================================================
	! read initial parameters

	open(2, file = 'optimal_paramun.dat', STATUS ='OLD')
		do j = 1, n
			read(2,*), xoptimal(j)
		end do
	close(2)

	! read initial guess of T
	open(2, file = 'optimal_techun.dat', STATUS ='OLD')
		do j = 1, NN
			read(2,*), T(j)
		end do
	close(2)

	! read initial guess of lambda
	open(2, file = 'optimal_lambdaun.dat', STATUS ='OLD')
		do j = 1, NN
			read(2,*), lambda(j)
		end do
	close(2)

! ===================================================================================
! calculate objective function at optimal and store relevant data 

	call calc_optimal(NN, n, nobs, theta, sigmaa, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, lambda, succt, xoptimal, obj, XTdata, XToptimal, residuals, obsimpexp)

! ===================================================================================
! estimate matrix of derivatives with respect to parameters--d(X_ni/(X_nX_i))/d(xoptimal)
	
	call calc_derivatives(NN, n, nobs, theta, sigmaa, H, w, wpop, XT99, distance, contig, lang, agree, uswage, T, lambda, XToptimal, xoptimal, Xderivatives) 

! ===================================================================================
! calculate variance of parameters 
	
!	case 1: no cluster
! ------------------------------------------------------
	call parameter_var(n, nobs, Xderivatives, residuals, varparam)

!   print parameters and corresponding standard errors:
	print *, " =================================== "
	print *, "parameter standard error  with clustering (transformed parameters)"
	print *, "d1     ", xoptimal(1), varparam(1,1)**.5
	print *, "d2     ", xoptimal(2), varparam(2,2)**.5
	print *, "d3     ", xoptimal(3), varparam(3,3)**.5
	print *, "b      ", xoptimal(4), varparam(4,4)**.5
	print *, "l      ", xoptimal(5), varparam(5,5)**.5
	print *, "tagree ", xoptimal(6), varparam(6,6)**.5
	print *, "alpha  ", 1./(xoptimal(7) + 1), varparam(7,7)**.5
	print *, "thetab ", xoptimal(8) + 1., varparam(8,8)**.5
	print *, "sigmab ", ((xoptimal(9)/(xoptimal(9)+1))*(xoptimal(8) + 1.) + 1.), varparam(9,9)**.5
	print *, " "

	!	case 2: cluster by importer and exporter
	! ------------------------------------------------------

	call cluster_paramvar(NN, n, nobs, Xderivatives, residuals, obsimpexp, varparam)

	print *, " =================================== "
	print *, "parameter standard error  with clustering (transformed parameters)"
	print *, "d1     ", xoptimal(1), varparam(1,1)**.5
	print *, "d2     ", xoptimal(2), varparam(2,2)**.5
	print *, "d3     ", xoptimal(3), varparam(3,3)**.5
	print *, "b      ", xoptimal(4), varparam(4,4)**.5
	print *, "l      ", xoptimal(5), varparam(5,5)**.5
	print *, "tagree ", xoptimal(6), varparam(6,6)**.5
	print *, "alpha  ", 1./(xoptimal(7) + 1), varparam(7,7)**.5
	print *, "thetab ", xoptimal(8) + 1., varparam(8,8)**.5
	print *, "sigmab ", ((xoptimal(9)/(xoptimal(9)+1))*(xoptimal(8) + 1.) + 1.), varparam(9,9)**.5
	print *, " "

	! ===================================================================================
	! get the variance of parameters using the delta method (instead of the variance-covariance matrix of their transforms)
	
	call deltavar(n, xoptimal, varparam)

	print *, " =================================== "
	print *, "parameter standard error  with clustering"
	print *, "d1     ", xoptimal(1), varparam(1,1)**.5
	print *, "d2     ", xoptimal(2), varparam(2,2)**.5
	print *, "d3     ", xoptimal(3), varparam(3,3)**.5
	print *, "b      ", xoptimal(4), varparam(4,4)**.5
	print *, "l      ", xoptimal(5), varparam(5,5)**.5
	print *, "tagree ", xoptimal(6), varparam(6,6)**.5
	print *, "alpha  ", 1./(xoptimal(7) + 1), varparam(7,7)**.5
	print *, "thetab ", xoptimal(8) + 1., varparam(8,8)**.5
	print *, "sigmab ", ((xoptimal(9)/(xoptimal(9)+1))*(xoptimal(8) + 1.) + 1.), varparam(9,9)**.5
	print *, " "

	open(1, file = 'varparam.csv', STATUS ='REPLACE')
		write(1,*), "parameter, value, standard error"
		write(1,*), "d1,     ", xoptimal(1), ", ", (varparam(1,1)**(.5))
		write(1,*), "d2,     ", xoptimal(2), ", ", (varparam(2,2)**(.5))
		write(1,*), "d3,     ", xoptimal(3), ", ", (varparam(3,3)**(.5))
		write(1,*), "b ,     ", xoptimal(4), ", ", (varparam(4,4)**(.5))
		write(1,*), "l ,     ", xoptimal(5), ", ", (varparam(5,5)**(.5))
		write(1,*), "tagree, ", xoptimal(6), ", ", (varparam(6,6)**(.5))
		write(1,*), "alpha,  ", 1./(xoptimal(7) + 1), ", ", (varparam(7,7)**(.5))
		write(1,*), "thetab, ", xoptimal(8) + 1., ", ", (varparam(8,8)**(.5))
		write(1,*), "sigmab, ", ((xoptimal(9)/(xoptimal(9)+1))*(xoptimal(8) + 1.) + 1.), ", ", (varparam(9,9)**(.5))
	close(1)


	CALL SYSTEM_CLOCK(COUNT=endtime)

	print*, " "
	print*, "=========================================================================="
	print*, "Time: ", DBLE(endtime - startime)/(60*DBLE(hz)), " minutes"
	print*, " "
	print*, " " 

end program main

